import java.util.BitSet;
import java.util.Map;

class ComponentManager{
    Map<String, ComponentArray> componentArrays = new HashMap<>();
    Map<String, Integer> componentPosIndex = new HashMap<>();
    int componentPos = 0;

    public void registerComponent(String name){
        if (componentArrays.containsKey(name)){
            System.err.println("Component " + name + " is already registered");
            return;
        }
        componentArrays.put(name, new ComponentArray());
        componentPosIndex.put(name, componentPos++);
    }

    public void addComponentToEntity(String componentName, Object componentData, int entity){
        componentArrays.get(componentName).insertData(entity, componentData);
    }

    public void removeComponentFromEntity(String componentName, int entity){
        componentArrays.get(componentName).removeData(entity);
    }

    // Java does not allow reflective typing, so must cast this in the retrieving function.
    public Object getComponent(String componentType, int entity){
        return componentArrays.get(componentType).getData(entity);
    }

    public void entityDestroyed(int entity){
        for (String key : componentArrays.keySet()) {
            componentArrays.get(key).entityDestroyed(entity);
        }
    }

    public Integer getComponentIndex(String name){
        return componentPosIndex.get(name);
    }
}